package hol;

import java.util.ArrayList;
import java.util.List;

import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapred.JobConf;
import org.apache.spark.SparkConf;
import org.apache.spark.api.java.JavaPairRDD;
import org.apache.spark.api.java.JavaSparkContext;
import org.apache.spark.api.java.function.Function;
import org.apache.spark.sql.DataFrame;
import org.apache.spark.sql.Row;
import org.apache.spark.sql.hive.HiveContext;

import oracle.spatial.hadoop.vector.util.JobUtils;
import oracle.spatial.spark.vector.SparkRecordInfo;
import oracle.spatial.spark.vector.rdd.SpatialJavaRDD;
import oracle.spatial.spark.vector.recordinfoprovider.GeoJsonRecordInfoProvider;
import oracle.spatial.hadoop.vector.geojson.mapred.GeoJsonInputFormat;

public class SparkSQLFiltering {
    public SparkSQLFiltering() {
        super();
    }
    
    public static void main(String[] args) throws Exception{
               //Create a JobConf
               JobConf jobConf = new JobConf();
               //set input path
               JobUtils.setupInputPath(args[0], jobConf);
               
               // Create a Java Spark Context
               SparkConf conf = new SparkConf().setAppName("SparkSQLFiltering");
               JavaSparkContext sparkContext = new JavaSparkContext(conf);
               
               //load text file RDD
               JavaPairRDD<LongWritable, Text> textRDD = sparkContext.hadoopRDD(jobConf, GeoJsonInputFormat.class, LongWritable.class, Text.class);

               //list of GeoJSON properties to add to the record extra fields
               List<String> properties = new ArrayList<String>();
               properties.add("friends_count");
               properties.add("location");
               properties.add("followers_count");
               properties.add("user_id");

               //create RecordInfoProvider
               GeoJsonRecordInfoProvider recordInfoProvider = new GeoJsonRecordInfoProvider(properties, 2, 8307);
               
               //create a spatial RDD
               SpatialJavaRDD<SparkRecordInfo> spatialRDD = SpatialJavaRDD.fromJavaRDD(textRDD.values(), recordInfoProvider);

               HiveContext sqlContext = new HiveContext(sparkContext.sc());
               
               //get the spatial DataFrame from the SpatialRDD
               //the geometries are in GeoJSON format
               DataFrame spatialDataFrame = spatialRDD.createSpatialDataFrame(sqlContext, properties);
               
               // Register the DataFrame as a table.
               spatialDataFrame.registerTempTable("tweets");
               //register UDFs
               sqlContext.sql("create temporary function ST_Polygon as 'oracle.spatial.hadoop.vector.hive.ST_Polygon'");
               sqlContext.sql("create temporary function ST_Point as 'oracle.spatial.hadoop.vector.hive.ST_Point'");
               sqlContext.sql("create temporary function ST_Contains as 'oracle.spatial.hadoop.vector.hive.function.ST_Contains'");
               
               // SQL can be run over RDDs that have been registered as tables.
               StringBuffer query = new StringBuffer();
               
               query.append("SELECT geometry, friends_count, location, followers_count FROM tweets ");
               query.append("WHERE ST_Contains( ");
               query.append("  ST_Polygon('{\"type\": \"Polygon\",\"coordinates\": [[[-106, 25], [-106, 30], [-104, 30], [-104, 25], [-106, 25]]]}', 8307) ");
               query.append("  , ST_Point(geometry, 8307) ");
               query.append("  , 0.05)");
               query.append("  and followers_count > 50");

               DataFrame results = sqlContext.sql(query.toString());
               
               //Filter the tweets in a query window (somewhere in the north of Mexico) 
               //and with more than 50 followers.
               //Note that since the geometries are in GeoJSON format it is possible to create the ST_Point like
               //ST_Point(geometry, 8307)
               //instead of
               //ST_Point(geometry, 'oracle.spatial.hadoop.vector.hive.json.GeoJsonHiveRecordInfoProvider')

               List<String> filteredTweets = results.javaRDD().map(new Function<Row, String>() {
                 public String call(Row row) {
                         StringBuffer sb = new StringBuffer();
                         sb.append("Geometry: ");
                         sb.append(row.getString(0));
                         
                         sb.append("\nFriends count: ");
                         sb.append(row.getString(1));
                         sb.append("\nLocation: ");
                         sb.append(row.getString(2));
                         sb.append("\nFollowers count: ");
                         sb.append(row.getString(3));

                         return sb.toString();
                 }
               }).collect();

               //print the filtered tweets
               filteredTweets.forEach(tweet -> System.out.println("Tweet: "+tweet));
               
               //close context
               sparkContext.close();
       }

}
